### {c} TIG 2007-2009
#  selectionHideShow.rb
#  Adds various context-menu selection and hide/show tools in sub-menu
#  v1.1 Now tests with ".is_a?" to speed-up [after AlexM's advice].
###
require 'sketchup'
###
class Sketchup::Group
  def definition_count
    return self.entities.parent.instances.length
  end
end
###
class SelectionTool
###
def SelectionTool::group_miner(ents)
  ents.each{|e|
    if e.is_a? Sketchup::Group and e.definition_count > 1
      e.make_unique
      SelectionTool::group_miner(e.entities)
    end#if
  }
end
###
def SelectionTool::invertSelection() 
	Sketchup.active_model.start_operation("Invert Selection")
  model=Sketchup.active_model
  ss=model.selection 
  entities=model.active_entities 
  entities.each{|ent|
    SelectionTool::group_miner([ent])
    ss.toggle(ent)
  }
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowAll(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide All Selected")
	ss.each{|ent|
    SelectionTool::group_miner([ent])
		if ent.hidden?
		  status=ent.hidden=false
	  else
		  status=ent.hidden=true		
		end#if
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hide(ss)
	Sketchup.active_model.start_operation("Hide Selected")
	ss.each{|ent|
    SelectionTool::group_miner([ent])
		status=ent.hidden=true		
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::unhide(ss)
	Sketchup.active_model.start_operation("Un-Hide Selected")
	ss.each{|ent|
    SelectionTool::group_miner([ent])
		status=ent.hidden=false		
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowAllUNselected(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide All UN-Selected [Active]")
	model=Sketchup.active_model
  ents=model.active_entities### active NOT all
  ss=model.selection
  ents.each{|ent|
    SelectionTool::group_miner([ent])
		if ent.hidden?
		  status=ent.hidden=false
	  else
		  status=ent.hidden=true		
		end#if
  }
  ss.each{|ent|
    if ent.hidden?
		  status=ent.hidden=false
	  else
		  status=ent.hidden=true		
		end#if
  }
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowAllUNselectedALL(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide All UNselected-Selected [ALL]")
	model=Sketchup.active_model
  ents=model.entities### all
  ss=model.selection
   ents.each{|ent|
    SelectionTool::group_miner([ent])
		if ent.hidden?
		  status=ent.hidden=false
	  else
		  status=ent.hidden=true
		end#if
  }
  ss.each{|ent|
    if ent.hidden?
		  status=ent.hidden=false
	  else
		  status=ent.hidden=true
		end#if
  }
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowEdges(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide Selected Edges")
	ss.each{|ent|
		if ent.is_a? Sketchup::Edge
			if ent.hidden?
				status=ent.hidden=false
			else
				status=ent.hidden=true
			end#if
		end#if
		if ent.is_a? Sketchup::Group
      SelectionTool::group_miner([ent])
			hideEdges(ent.entities)
		end
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowFaces(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide Selected Faces")
	ss.each{|ent|
		if ent.is_a?(Sketchup::Face)
			if ent.hidden?
				status=ent.hidden=false
			else
				status=ent.hidden=true
			end#if
		end#if
		if ent.is_a? Sketchup::Group
      SelectionTool::group_miner([ent])
			hideFaces(ent.entities)
		end
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowGroups(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide Selected Groups")
	ss.each{|ent|
		if ent.is_a? Sketchup::Group
      SelectionTool::group_miner([ent])
			if ent.hidden?
				status=ent.hidden=false
			else
				status=ent.hidden=true
			end#if
		end#if
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::hideShowCompos(ss)
	Sketchup.active_model.start_operation("Hide/Un-Hide Selected Components")
	ss.each{|ent|
		if ent.is_a? Sketchup::ComponentInstance
			if ent.hidden?
				status=ent.hidden=false
			else
				status=ent.hidden=true
			end#if
		end#if
	}
	Sketchup.active_model.commit_operation
end
###
def SelectionTool::unHideAllActive()
  Sketchup.active_model.start_operation("Un-Hide All Active")
    Sketchup.active_model.active_entities.each{|e|e.hidden=false if e.hidden?}
  Sketchup.active_model.commit_operation
end#def
###
end#class
###
if not file_loaded?("#SelectionHideShow.rb")
	UI.add_context_menu_handler do |menu|
    menu.add_separator
		menu=menu.add_submenu("Selection/Hide/UnHide...")
    menu.add_item("Invert Selection"){SelectionTool::invertSelection()}
    menu.add_item("Hide All Selected"){SelectionTool::hide(Sketchup.active_model.selection)}
    menu.add_item("Un-Hide All Selected"){SelectionTool::unhide(Sketchup.active_model.selection)}
    menu.add_item("Un-Hide All Active"){SelectionTool::unHideAllActive()}
    menu.add_item("Toggle Hide/Un-Hide on All Selected"){SelectionTool::hideShowAll(Sketchup.active_model.selection)}
    menu.add_item("Toggle Hide/Un-Hide on All UN-Selected [Active]"){SelectionTool::hideShowAllUNselected(Sketchup.active_model.selection)}
    menu.add_item("Toggle Hide/Un-Hide on All UN-Selected [ALL]"){SelectionTool::hideShowAllUNselectedALL(Sketchup.active_model.selection)}
    menu.add_item("Toggle Hide/Un-Hide on Selected Edges"){SelectionTool::hideShowEdges(Sketchup.active_model.selection)}
	menu.add_item("Toggle Hide/Un-Hide on Selected Faces"){SelectionTool::hideShowFaces(Sketchup.active_model.selection)}
	menu.add_item("Toggle Hide/Un-Hide on Selected Groups"){SelectionTool::hideShowGroups(Sketchup.active_model.selection)}
	menu.add_item("Toggle Hide/Un-Hide on Selected Components"){SelectionTool::hideShowCompos(Sketchup.active_model.selection)}
  menu.add_item("Toggle Hidden Geometry on/off"){Sketchup.send_action( "viewShowHidden:")}
	end#menu
	file_loaded("#SelectionHideShow.rb")
end#if
###
